@@ -46,7 +46,7 @@ from contextlib import contextmanager
 import msgpack
 import pylibmc
 
-VERSION = "0.6.6"
+VERSION = "0.6.8"
 LOCK_SH = 0
 LOCK_EX = 1
 RUN_PATH = "/run/starrydb/"
@@ -181,8 +181,9 @@ class LibMemcachedClient(pylibmc.ThreadMappedPool):
             return mc.delete(key)
 
 class StarryServer(object):
-    def __init__(self, address=SERVER_ADDRESS, depots=[], cache_size=0, log_file=None, rw_ratio=5):
+    def __init__(self, address=SERVER_ADDRESS, depots=[], cache_address=CACHE_SOCKET, cache_size=0, log_file=None, rw_ratio=5):
         self.address = address
+        self.cache = cache_address
         self.cursor = 0
         self.depots = {}
         self.logger = StarryLog(log_file) if log_file is not None else None
@@ -193,16 +194,16 @@ class StarryServer(object):
 
         if int(cache_size) > 0:
             try:
-                subprocess.check_output(f"fuser -k {CACHE_SOCKET}", shell=True, stderr=subprocess.STDOUT, universal_newlines=True)
+                subprocess.check_output("kill -9 `ps axf | grep " + self.cache + " | grep -v grep | awk '{print $1}'`", shell=True, stderr=subprocess.STDOUT, universal_newlines=True)
             except:
                 pass
 
             try:
-                os.system(f"memcached -d -u root -a 0777 -D : -C -m {int(cache_size)} -s {CACHE_SOCKET}")
+                os.system(f"memcached -d -u root -a 0777 -D : -C -m {int(cache_size)} -s {self.cache}")
 
                 time.sleep(1)
 
-                self.memcached = LibMemcachedClient(CACHE_SOCKET)
+                self.memcached = LibMemcachedClient(self.cache)
             except Exception as e:
                 self.logger.error(f"Caught exception: {e.__doc__}({e})")
 
@@ -220,6 +221,9 @@ class StarryServer(object):
                 if depot.get("cache", False) is True:
                     self.depots[depot_path].enableCache()
 
+                if depot.get("flush", False) is True:
+                    self.depots[depot_path].enableFlush()
+
                 self.cursor += 1
 
     def __del__(self):
@@ -232,15 +236,10 @@ class StarryServer(object):
                 pass
 
             try:
-                subprocess.check_output(f"fuser -k {CACHE_SOCKET}", shell=True, stderr=subprocess.STDOUT, universal_newlines=True)
+                subprocess.check_output("kill -9 `ps axf | grep " + self.cache + " | grep -v grep | awk '{print $1}'`", shell=True, stderr=subprocess.STDOUT, universal_newlines=True)
             except:
                 pass
 
-        try:
-            shutil.rmtree(RUN_PATH, ignore_errors=True)
-        except:
-            pass
-
     def start(self, max_workers=512, backlog=512, authkey=b'starry'):
         if pathlib.Path(str(self.address)).is_socket():
             try:
@@ -288,6 +287,7 @@ class StarryDepot(object):
         self.index = index
         self.memcached = memcached
         self.cache = None
+        self.flush = False
         self.bak = False
         self.thread_cond = threading.Condition()
         self.thread_lock = threading.RLock()
@@ -349,23 +349,48 @@ class StarryDepot(object):
         except:
             pass
 
+    def enableFlush(self):
+        self.flush = True
+
+        return (True, "")
+
+    def disableFlush(self):
+        self.flush = False
+
+        return (True, "")
+
     def enableCache(self):
         self.cache = self.memcached
 
+        return (True, "")
+
     def disableCache(self):
+        mark = {"mode": "r", "members": [None]}
+
+        with self._lock(mark):
+            for elem in self.database.glob("*"):
+                if elem.suffix == ".mpb":
+                    self._deleteCache(elem.stem)
+
         self.cache = None
 
-    def enableBackup(self):
-        self.bak = True
+        return (True, "")
 
+    def enableBackup(self):
         if not self.database.joinpath(".backup").is_dir():
             self.database.joinpath(".backup").mkdir(exist_ok=True)
 
             self.backup()
 
+        self.bak = True
+
+        return (True, "")
+
     def disableBackup(self):
         self.bak = False
 
+        return (True, "")
+
     def get(self, key):
         if type(key) is not list:
             key = [key]
@@ -752,7 +777,7 @@ class StarryDepot(object):
 
                 return (False, f"Caught exception: {e.__doc__}({e})")
 
-        (ret, msg) = _writeFile(file, value)
+        (ret, msg) = _writeFile(file, value, self.flush)
         if ret is False:
             self._recover(key)
             return (ret, msg)
@@ -813,7 +838,7 @@ class StarryDepot(object):
 
                 return (False, f"Caught exception: {e.__doc__}({e})")
 
-        (ret, msg) = _writeFile(file, value)
+        (ret, msg) = _writeFile(file, value, self.flush)
         if ret is False:
             self._recover(key)
             return (ret, msg)
@@ -906,7 +931,7 @@ class StarryDepot(object):
         except Exception as e:
             return (False, f"Caught exception: {e.__doc__}({e})")
 
-        (ret, msg) = _writeFile(file, value)
+        (ret, msg) = _writeFile(file, value, self.flush)
         if ret is False:
             self._recover(key)
             return (ret, msg)
@@ -938,7 +963,7 @@ class StarryDepot(object):
         if path is None:
             path = self.database.joinpath(".backup")
 
-        return _copyFile(file, path.joinpath(f"{key}.mpb"))
+        return _copyFile(file, path.joinpath(f"{key}.mpb"), self.flush)
 
     def _recover(self, key):
         backup_file = self.database.joinpath(".backup", f"{key}.mpb")
@@ -967,7 +992,7 @@ class StarryDepot(object):
 
             return (False, f"Caught exception: {e.__doc__}({e})")
 
-        (ret, msg) = _copyFile(backup_file, self.database.joinpath(f"{key}.mpb"))
+        (ret, msg) = _copyFile(backup_file, self.database.joinpath(f"{key}.mpb"), self.flush)
         if ret is False:
             return (ret, msg)
 
@@ -999,7 +1024,7 @@ class StarryDepot(object):
                     if ret is False:
                         return (ret, msg)
 
-        (ret, msg) = _copyFile(source_file, file)
+        (ret, msg) = _copyFile(source_file, file, self.flush)
         if ret is False:
             if not backup_file.exists():
                 try:
@@ -1342,14 +1367,15 @@ def _readFile(src):
     except Exception as e:
         return (False, f"Caught exception: {e.__doc__}({e})")
 
-def _writeFile(dst, value):
+def _writeFile(dst, value, flush=False):
     try:
         with dst.open(mode="wb", buffering=0) as descriptor:
             descriptor.write(value)
             descriptor.truncate()
 
-            descriptor.flush()
-            os.fdatasync(descriptor.fileno())
+            if flush is True:
+                descriptor.flush()
+                os.fdatasync(descriptor.fileno())
 
         if len(value) != dst.stat().st_size:
             return (False, "No space left on device.")
@@ -1358,7 +1384,7 @@ def _writeFile(dst, value):
 
     return (True, "")
 
-def _copyFile(src, dst):
+def _copyFile(src, dst, flush=False):
     (ret, value) = _readFile(src)
     if ret is False:
         return (ret, value)
@@ -1366,7 +1392,7 @@ def _copyFile(src, dst):
     if dst.exists():
         tmp_path = dst.with_suffix(".mpb.tmp")
 
-        (ret, msg) = _writeFile(tmp_path, value)
+        (ret, msg) = _writeFile(tmp_path, value, flush)
         if ret is False:
             try:
                 tmp_path.unlink()
@@ -1383,11 +1409,11 @@ def _copyFile(src, dst):
             except:
                 pass
 
-            (ret, msg) = _writeFile(dst, value)
+            (ret, msg) = _writeFile(dst, value, flush)
             if ret is False:
                 return (ret, msg)
     else:
-        (ret, msg) = _writeFile(dst, value)
+        (ret, msg) = _writeFile(dst, value, flush)
         if ret is False:
             try:
                 dst.unlink()
